@@ -1342,7 +1342,8 @@ vt_set_text(struct vt_em *vt)
 {
   char *p;
   int i;
-
+  char *utf8;
+  
   if (vt->change_my_name) {
     p = strchr(vt->arg.txt.args_mem, ';');
     if (p) {
@@ -1368,8 +1369,43 @@ vt_set_text(struct vt_em *vt)
       default:			/* dont care, piss off */
 	return;
       }
-      /* Order changed to reflect the changed arg. order */
-      vt->change_my_name(vt->user_data, p, i);
+
+      /* XPROPERTY sets an XA_STRING in gnome-terminal and must
+       * therefore be Latin-1 unless gnome-terminal is on crack.
+       * We'll assume the title/iconname are in locale encoding, since
+       * AFAIK they aren't tagged for encoding. xterm seems to treat
+       * them as locale encoding.
+       */
+
+      if (i == VTTITLE_XPROPERTY)
+        {
+          const char *s;
+          GString *str;
+
+          str = g_string_new ("");
+          
+          s = p;
+          while (*s)
+            {
+              g_string_append_unichar (str, *s);
+              ++s;
+            }
+
+          utf8 = g_string_free (str, FALSE);
+        }
+      else
+        {
+          utf8 = g_locale_to_utf8 (p, -1, NULL, NULL, NULL);
+          /* utf8 will be NULL if the title was not valid
+           * in the locale encoding
+           */
+        }
+      
+      if (utf8)
+        {
+          vt->change_my_name (vt->user_data, utf8, i);
+          g_free (utf8);
+        }
     }
   }
 }
